#include <Servo.h>

// Pinbelegung Stellpult //
// PIN D2  - Signalleitung 1
// PIN D3  - Signalleitung 2
// PIN D4  - Signalleitung 3
// PIN D5  - Signalleitung 4
// PIN D6  - 
// PIN D7  -  
// PIN D8  - 
// PIN D9  - 
// PIN D10 - Servo Ra12
// PIN D11 - Servo Vorsignal Hp0/Hp1
// PIN D12 - Servo Vorsignal Hp2
// PIN D13 - Servo Haupsignal Hp0/Hp1
// PIN A0  - Servo Hauptsignal Hp2
// PIN A1  - 
// PIN A2  - Eingang vom vorausliegenden Hauptsignal - Signalleitung 1
// PIN A3  - Eingang vom vorausliegenden Hauptsignal - Signalleitung 2
// PIN A4  - Eingang vom vorausliegenden Hauptsignal - Signalleitung 3
// PIN A5  - Eingang vom vorausliegenden Hauptsignal - Signalleitung 4
// PIN A6  -
// PIN A7  -

// Konstanten
/* PINs für Buchse 1 - Steuerung von Stellpult */
const char in_Buchse1Leitung1 = 2;
const char in_Buchse1Leitung2 = 3;
const char in_Buchse1Leitung3 = 4;
const char in_Buchse1Leitung4 = 5;

/* PINs für Buchse 2 - Signal vom vorausliegenden Hauptsignal  */
const char in_Buchse2Leitung1 = A2;
const char in_Buchse2Leitung2 = A3;
const char in_Buchse2Leitung3 = A4;
const char in_Buchse2Leitung4 = A5;

/* PINs für Servoansteuerung */
const int out_ServoH1 = 10;
const int out_ServoH2 = 11;
const int out_ServoV1 = 12;
const int out_ServoV2 = 13;
const int out_ServoRa12 = 14;

/* Signalbilder */
const char c_SignalbildHp0  = 'Hp0';
const char c_SignalbildHp1  = 'Hp1';
const char c_SignalbildHp2  = 'Hp2';
const char c_SignalbildV60  = 'V60';
const char c_SignalbildV80  = 'V80';
const char c_SignalbildV100 = 'V100';
const char c_SignalbildZs1  = 'Zs1';
const char c_SignalbildRa12 = 'Ra12';
const char c_SignalbildVr0  = 'Vr0';
const char c_SignalbildVr1  = 'Vr1';
const char c_SignalbildVr2  = 'Vr2';

// Variablen
/* Status der Signalleitung von Stellpult */
int i_StateLeitung1;
int i_StateLeitung2;
int i_StateLeitung3;
int i_StateLeitung4;

/* Status der Signalleitung vom vorausliegenden Hauptsignal  */
int i_StateBuchse2Leitung1;
int i_StateBuchse2Leitung2;
int i_StateBuchse2Leitung3;
int i_StateBuchse2Leitung4;

/* Signalbild Hauptsignal */
char HpBild;
/* Signalbild Vorsignaö */
char VrBild;

// Servoobjekte
Servo ServoHauptsignal1;
Servo ServoHauptsignal2;
Servo ServoVorsignal1;
Servo ServoVorsignal2;
Servo ServoRa12;

// ************************************************************** //
// Signalbilder setzen
// ************************************************************** //
void setHp0( ) {
  ServoHauptsignal1.write(30);
  ServoHauptsignal2.write(30);
  ServoRa12.write(30);
}

void setHp1( ) {
  ServoHauptsignal1.write(120);
  ServoHauptsignal2.write(30);
}

void setHp2( ) {
  ServoHauptsignal1.write(120);
  ServoHauptsignal2.write(120);
}

void setRa12( ) {
  ServoRa12.write(120);
}

void setVr0( ) {
  ServoVorsignal1.write(30);
  ServoVorsignal2.write(30);
}

void setVr1( ) {
  ServoVorsignal1.write(120);
  ServoVorsignal2.write(30);
}

void setVr2( ) {
  ServoVorsignal1.write(120);
  ServoVorsignal2.write(120);
}

void setHpSignal( ) {

  switch (HpBild ) {
    case c_SignalbildHp0:
      setHp0( );
      break;
    case c_SignalbildHp1:
      setHp1( );
      break;
    case c_SignalbildHp2:
      setHp2( );
      break;
    case 'Ra12':
      setRa12( );
      break;
    default:
      break;
  }
}

void setVrSignal( ) {
  switch (VrBild) {
    case c_SignalbildVr0:
      setVr0( );
      break;
    case c_SignalbildVr1:
      setVr1( );
      break;
    case c_SignalbildVr2:
      setVr2( );
      break;
    default:
      break;
  }
}

void attachServos( ) {
  ServoHauptsignal1.attach(out_ServoH1);
  ServoHauptsignal2.attach(out_ServoH2);
  ServoVorsignal1.attach(out_ServoV1);
  ServoVorsignal2.attach(out_ServoV2);
  ServoRa12.attach(out_ServoRa12);
}

void readInput( ) {
  // Hauptsignal
  i_StateLeitung1 = digitalRead(in_Buchse1Leitung1);
  i_StateLeitung2 = digitalRead(in_Buchse1Leitung2);
  i_StateLeitung3 = digitalRead(in_Buchse1Leitung3);
  i_StateLeitung4 = digitalRead(in_Buchse1Leitung4);

  // Vorsignal
  i_StateBuchse2Leitung1  = digitalRead(in_Buchse2Leitung1);
  i_StateBuchse2Leitung2  = digitalRead(in_Buchse2Leitung2);
  i_StateBuchse2Leitung3  = digitalRead(in_Buchse2Leitung3);
  i_StateBuchse2Leitung4  = digitalRead(in_Buchse2Leitung4);
}

// ************************************************************** //
// ************************************************************** //
// ************************************************************** //
void setup() {
  // ************************************************************** //

  attachServos( );  /* Servos initialisieren */
  setHp0( );        /* initial alles auf Halt */ 
  setVr0( );        /* initial alles auf Halt */

}

// ************************************************************** //
// ************************************************************** //
// ************************************************************** //
void loop() {

  // ************************************************************** //
  // Eingangssignal lesen
  // ************************************************************** //

  readInput( );

  // ************************************************************** //
  // Hauptsignalbild und Ra12 ermitteln
  // ************************************************************** //

  // Hp1
  if ( i_StateLeitung1 == HIGH and
       i_StateLeitung2 == HIGH and
       i_StateLeitung3 == HIGH ) {
    HpBild = c_SignalbildHp1;
  }

  // Hp2
  if ( i_StateLeitung1 == LOW and
       i_StateLeitung2 == LOW and
       i_StateLeitung3 == HIGH ) {
    HpBild = c_SignalbildHp1;
  }

  // Ra12
  if ( i_StateLeitung1 == LOW and
       i_StateLeitung2 == HIGH and
       i_StateLeitung3 == LOW ) {
    HpBild = c_SignalbildRa12;
  }

  // Hp0
  if ( i_StateLeitung1 == LOW  and
       i_StateLeitung2 == HIGH and
       i_StateLeitung3 == HIGH ) {
    HpBild = c_SignalbildHp0;
  }


  // ************************************************************** //
  // Vorsignalbild ermitteln
  // ************************************************************** //

  // Vr1
  if ( i_StateBuchse2Leitung1 == HIGH and
       i_StateBuchse2Leitung2 == HIGH and
       i_StateBuchse2Leitung3 == HIGH ) {
    VrBild = c_SignalbildVr1;
  }

  // Vr2
  if ( i_StateBuchse2Leitung1 == LOW and
       i_StateBuchse2Leitung2 == LOW and
       i_StateBuchse2Leitung3 == HIGH ) {
    VrBild = c_SignalbildVr1;
  }

  // Vr0
  if ( i_StateBuchse2Leitung1 == LOW  and
       i_StateBuchse2Leitung2 == HIGH and
       i_StateBuchse2Leitung3 == HIGH ) {
    VrBild = c_SignalbildVr0;
  }

  // ************************************************************** //
  // Signalbild setzen
  // ************************************************************** //

  // Hauptsignalbild und Ra12
  setHpSignal( );

  // Vorsignalbild
  setVrSignal( );
}

